/*******************************************************************
 *
 *	File:		CArlo.h
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __CARLO_H
#define __CARLO_H

#include "..\boot2nd\boot2nd_if.h"

#define KDefaultSeries7KernelLoadAdr	((TUint*) 0xC0208000)
#define KDefaultSeries7InitrdLoadAdr	((TUint*) 0xC0400000)
#define KDefaultSeries7ArchNumber		((TInt) 64)

#define KDefaultSeries5mxKernelLoadAdr	((TUint*) 0xC0048000)
#define KDefaultSeries5mxInitrdLoadAdr	((TUint*) 0xC0400000)
#define KDefaultSeries5mxArchNumber		((TInt) 60)

class CPalette;
class TRgb;

/*
 * Pagetable size is: 3 * PAGES_SIZE * SIZEOF(UINT) 
 */
#define BOOT_STRING_SIZE			64
#define NR_PAGE_TABLE_ENTRIES		((PAGE_SIZE << 1) - 2)

class RArLo;
class CArloBase;

class ArloFactory
{
public:
	static CArloBase* CreateArloLC(RArLo& aArlo, const TDesC& aMachineName, RFs& aFs, RWsSession& aWsSession);
};


class CArloBase : public CBase
{
public:
	virtual void ConstructL(RFs* aFs, RWsSession* aWsSession);
	~CArloBase();

	TUint32	ArloLinear2Physical(TUint32 aLinearAdr) {return PlatLinear2Physical(aLinearAdr);}
	TUint	ArloExamine(TUint32 aAddressToExamine) {return PlatExamine(aAddressToExamine);}
	void	ArloBoot();
	TUint	ArloGetProcId() {return GetProcId();}
	void	SetBootString(TAny* aBootString, TInt aLength);
	void	AddPagesL(TInt aBlocks, TUint32 aLoadAddr);
	const TDesC& MachineName() {return iMachineName;}
	TBool	HasLoadedFile(){return iArloPageTableIndex > 2;}
	void	Report();
	void	LoadFileL(const TDesC& aFileName, TUint& aFileSize, TUint32& aLoadAdr, TUint& aNoOfBlocks);

	virtual TUint*		DefaultKernelLoadAdr() {return 0;}
	virtual TUint*		DefaultInitrdLoadAdr() {return 0;}
	virtual TInt		DefaultArchNumber() {return -1;}
	boot1st_param_t*	Boot1stParam() {return iBoot1stParam;}
	TUint				NearestColourIndex(TRgb aRgbColour);
	void				UseBoot2nd(TBool aUseBoot2nd){iUseBoot2nd = aUseBoot2nd;}

protected:
	CArloBase(RArLo& aArlo);		// only ArloFactory constructs us
	virtual void SetupArloPageTableL();

	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr) = 0;
	virtual TUint	PlatExamine(TUint32 aAddressToExamine) = 0;
	virtual void	PlatBoot(TUint32 aPhysMapAdr) = 0;
	virtual TUint	GetProcId();
	virtual void	SetupVideoParametersL(boot1st_param_t* /*aArloParam*/){};

protected:
	TBool				iUseBoot2nd;
	TBool				iHasSetupArloPageTable;
	TInt				iArloPageTableIndex;
	TText8*				iBootString;		// we own this
	TUint32*			iArloPageTable;
	RArLo&				iArlo;
	TInfoName			iMachineName;
	RFs*				iFs;
	RWsSession*			iWsSession;
	boot1st_param_t*	iBoot1stParam;
	CPalette*			iPalette;
	friend class ArloFactory;

private:
	TText8*				iTable;				// we own this
};


class CArloSeries5 : public CArloBase
{
protected:
	CArloSeries5(RArLo& aArlo) : CArloBase(aArlo){} 
	virtual TUint	PlatExamine(TUint32 aAddressToExamine);
	virtual void	PlatBoot(TUint32 aPhysMapAdr);
	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr);
	friend class ArloFactory;
};


class CArloArm4 : public CArloBase
{
protected:
	CArloArm4(RArLo& aArlo) : CArloBase(aArlo){} 
	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr);
	virtual TUint	PlatExamine(TUint32 aAddressToExamine);
	virtual void SetupArloPageTableL();
	virtual void SetupVideoParametersL(boot1st_param_t* aArloParam);
	void LoadBoot2ndL();
};

class CArloSeries5Mx : public CArloArm4
{
public:
	virtual TUint*	DefaultKernelLoadAdr() {return KDefaultSeries5mxKernelLoadAdr;}
	virtual TUint*	DefaultInitrdLoadAdr() {return KDefaultSeries5mxInitrdLoadAdr;}
	virtual TInt	DefaultArchNumber() {return KDefaultSeries5mxArchNumber;}

protected:
	CArloSeries5Mx(RArLo& aArlo) : CArloArm4(aArlo){} 
	virtual void	PlatBoot(TUint32 aPhysMapAdr);
	friend class ArloFactory;
	virtual void SetupArloPageTableL();
};

class CArloSeries7 : public CArloArm4
{
public:
	virtual TUint*	DefaultKernelLoadAdr() {return KDefaultSeries7KernelLoadAdr;}
	virtual TUint*	DefaultInitrdLoadAdr() {return KDefaultSeries7InitrdLoadAdr;}
	virtual TInt	DefaultArchNumber() {return KDefaultSeries7ArchNumber;}

protected:
	CArloSeries7(RArLo& aArlo) : CArloArm4(aArlo){} 

	virtual void SetupVideoParametersL(boot1st_param_t* aArloParam);
	virtual void PlatBoot(TUint32 aPhysMapAdr);
	friend class ArloFactory;

};


class CArloWins : public CArloBase
{
protected:
	CArloWins(RArLo& aArlo) : CArloBase(aArlo){} 
	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr){return aLinearAdr;}
	virtual TUint	PlatExamine(TUint32 aAddressToExamine){ return aAddressToExamine;}
	virtual void	PlatBoot(TUint32 aPhysMapAdr);
	friend class ArloFactory;
};





#endif			/* __CARLO_H */
