/*******************************************************************
 *
 *	File:		CArloConfig.h
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __C_ARLO_CONFIG__H
#define __C_ARLO_CONFIG__H

const TInt		KMaxConfigTextString = 64;
const TInt		KMaxBootTextString = 1024;
typedef TBuf<KMaxConfigTextString>	TKernelFileName;
typedef TBuf<KMaxConfigTextString>	TInitrdFileName;
typedef TBuf<KMaxConfigTextString>	TDescription;
typedef TBuf<KMaxBootTextString>	TBootString;

class TArloBootItem
{
public:
	TKernelFileName		iKernelFileName;
	TUint				iKernelLoadAdr;
	TInitrdFileName		iInitrdFileName;
	TUint				iInitrdLoadAdr;
	TDescription		iDescription;
	TBootString			iBootString;
	TInt				iArchNumber;
	TUint				iForegroundColour;
	TUint				iBackgroundColour;
	TBool				iHasColoursSet;
};


class CArloConfig : public CBase
{
public:
	~CArloConfig();
	static CArloConfig* ReadConfigFileLC();
	
	TInt BootItems(){return iArloBootItems->Count();}
	TInt DefaultBootItem(){return iDefaultBootItem;}
	TInt Timeout(){return iTimeout;}
	TBool UseBoot2nd(){return iUseBoot2nd;}

	const TArloBootItem& BootItemL(TInt aItem){return iArloBootItems->At(aItem);}
	const TDesC& MachineName() {return iMachineName;}
	const TDesC& UsedConfigFile() {return iConfigFile;}

protected:
	CArloConfig();
	void ConstructL();
	void ReadConfigFileL(RFs& aFs, const TDesC& aFileName);
	void ParseLine(const TDesC& aLine);
	void ParseItems(const TDesC& aToken, const TDesC&  aValue);
	void ParseFileNameAndLoadAdr(const TDesC& aString, TDes& aFileName, TUint& aLoadAdr);
	TBool GetNumber(TLex& aLex, TUint& aNumber);
	void NewBootItem(const TDesC& aBootItem);
	void SetDefaultBootItem();

	void Defaults();

	TBool			iUseBoot2nd;
	TInt			iTimeout;
	TInt			iDefaultBootItem;
	TDescription	iDefaultBootItemString;
	TInfoName		iMachineName;
	TInfoName		iConfigFile;
	CArrayFixFlat<TArloBootItem>*	iArloBootItems;
};


#endif			/* __C_ARLO_CONFIG__H */